<?php

namespace ZRDN;

/**
 * Basic Authentication Class
 *
 * Basic Authentication  handler for the JSON API, used for zip recipes plugin purposes
 *
 * PHP version 5.3
 * Depends: WP REST API, Plugin Dependencies
 *
 * @package    Zip Recipes
 * @author     Mudassar Ali <sahil_bwp@yahoo.com>
 * @copyright  2017 Gezim Hoxha
 * @version 1.0
 */
class ZRDN_Basic_Auth_Controller {

    public function __construct() {
        add_filter('determine_current_user', array($this, 'json_basic_auth_handler'), 20);
        add_filter('rest_authentication_errors', array($this, 'json_basic_auth_error'));
    }

    public function json_basic_auth_handler($user) {
        global $wp_json_basic_auth_error;
        $wp_json_basic_auth_error = null;
        // Don't authenticate twice
        if (!empty($user)) {
            return $user;
        }
        // Check that we're trying to authenticate
        if (!isset($_SERVER['PHP_AUTH_USER'])) {
            return $user;
        }
        $username = $_SERVER['PHP_AUTH_USER'];
        $password = $_SERVER['PHP_AUTH_PW'];
        /**
         * In multi-site, wp_authenticate_spam_check filter is run on authentication. This filter calls
         * get_currentuserinfo which in turn calls the determine_current_user filter. This leads to infinite
         * recursion and a stack overflow unless the current function is removed from the determine_current_user
         * filter during authentication.
         */
        remove_filter('determine_current_user', 'json_basic_auth_handler', 20);
        $user = wp_authenticate($username, $password);
        add_filter('determine_current_user', 'json_basic_auth_handler', 20);
        if (is_wp_error($user)) {
            $wp_json_basic_auth_error = $user;
            return null;
        }
        $wp_json_basic_auth_error = true;
        return $user->ID;
    }

    public function json_basic_auth_error($error) {
        // Passthrough other errors
        if (!empty($error)) {
            return $error;
        }
        global $wp_json_basic_auth_error;
        return $wp_json_basic_auth_error;
    }

}

new ZRDN_Basic_Auth_Controller();
